package com.bytedance.article.common.extensions

/**
 * Collection扩展函数
 *
 * @author zhanghao
 * @date 2018/4/16
 * @email zhanghao@bytedance.com
 */

/**
 * 集合是否为空，等价于CollectionUtils.isEmpty()
 */
fun <T> Collection<T>?.isNullOrEmpty():Boolean = this == null || this.isEmpty()

/**
 * 集合是否不为空，等价于!CollectionUtils.isEmpty()，注意与Collection<*>.isNotEmpty的区别
 */
fun <T> Collection<T>?.isNotEmpty():Boolean = !this.isNullOrEmpty()

/**
 * 集合不为空，则执行后面的代码块
 */
inline fun <T, R> Collection<T>?.ifNotEmpty(block: (Collection<T>) -> R): R? = if (this.isNotEmpty()) {
    block(this as Collection<T>)
} else null

/**
 * 集合为空，则执行后面的代码块
 */
inline fun <T, R> Collection<T>?.ifEmpty(block: (Collection<T>?) -> R): R? = if (this.isNullOrEmpty()) {
    block(this)
} else null