package com.bytedance.article.common.extensions

/**
 * String扩展函数
 *
 * @author zhanghao
 * @date 2018/4/16
 * @email zhanghao@bytedance.com
 */

/**
 * 字符串是否不为空，等价于!TextUtils.isEmpty()，注意与CharSequence.isNotEmpty的区别
 */
fun CharSequence?.isNotEmpty():Boolean = !this.isNullOrEmpty()

/**
 * 字符串不为空，则执行后面的代码块
 */
inline fun <R> CharSequence?.ifNotEmpty(block: (CharSequence) -> R): R? = if (this.isNotEmpty()) {
    block(this as CharSequence)
} else null

/**
 * 字符串不为空，则执行后面的代码块
 */
inline fun <R> CharSequence?.ifEmpty(block: (CharSequence?) -> R): R? = if (this.isNullOrEmpty()) {
    block(this)
} else null