package com.ss.android.article.base.ui

import android.content.Context
import android.support.annotation.StringRes
import android.support.v7.widget.AppCompatImageView
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.airbnb.lottie.LottieAnimationView
import com.bytedance.article.common.utils.ViewUtils
import com.bytedance.common.utility.UIUtils
import com.ss.android.article.base.R

/**
 * 详情页赞同/反对Layout
 *
 * @author zhanghao
 * @date 2018/5/15
 * @email zhanghao@bytedance.com
 */
class AnswerDetailDiggBuryLayout : LinearLayout {

    private lateinit var mDiggLayout: LinearLayout
    private lateinit var mDiggView: LottieAnimationView
    private lateinit var mDiggTv: TextView
    private lateinit var mBuryLayout: LinearLayout
    private lateinit var mBuryView: ImageView

    private var mDrawableWidth = UIUtils.dip2Px(context, 40f).toInt()
    private var mDrawableHeight = UIUtils.dip2Px(context, 40f).toInt()
    private var mDiggLeftPadding = UIUtils.dip2Px(context, 8f).toInt()
    private var mDiggTvMinWidth = UIUtils.dip2Px(context, 32f).toInt()
    private var mBuryWidth = UIUtils.dip2Px(context, 24f).toInt()
    private var mBuryHeight = UIUtils.dip2Px(context, 24f).toInt()
    private var mBuryLeftPadding = UIUtils.dip2Px(context, 8f).toInt()
    private var mBuryRightPadding = UIUtils.dip2Px(context, 16f).toInt()
    private var mDigitalTextSize = UIUtils.sp2px(context, 16f).toInt()
    private var mWordTextSize = UIUtils.sp2px(context, 14f).toInt()

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context, attrs, defStyleAttr)
    }

    private fun init(context: Context, attrs: AttributeSet?, defStyleAttr: Int) {
        //允许动画超出边界
        clipToPadding = false
        clipChildren = false
        orientation = HORIZONTAL
        gravity = Gravity.CENTER_VERTICAL

        initDiggLayout(context)
        initBuryLayout()
    }

    private fun initDiggLayout(context: Context) {
        val diggLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.MATCH_PARENT)
        mDiggLayout = LinearLayout(context)
        mDiggLayout.orientation = HORIZONTAL
        mDiggLayout.gravity = Gravity.CENTER_VERTICAL
        mDiggLayout.setPadding(mDiggLeftPadding, 0, 0, 0)

        mDiggView = LottieAnimationView(context)
        mDiggView.setAnimation("un_dynamic.json", LottieAnimationView.CacheStrategy.Weak)
        val diggViewLayoutParams = ViewGroup.MarginLayoutParams(mDrawableWidth, mDrawableHeight)
        mDiggLayout.addView(mDiggView, diggViewLayoutParams)

        mDiggTv = TextView(context)
        mDiggTv.setTextColor(context.resources.getColorStateList(R.color.c8_c3_selector))
        mDiggTv.setTextSize(TypedValue.COMPLEX_UNIT_PX, mDigitalTextSize.toFloat())
        mDiggTv.minWidth = mDiggTvMinWidth
        mDiggTv.gravity = Gravity.CENTER_HORIZONTAL
        val diggTextTvLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.WRAP_CONTENT)
        mDiggLayout.addView(mDiggTv, diggTextTvLayoutParams)

        addView(mDiggLayout, diggLayoutParams)
        requestLayout()
    }

    private fun initBuryLayout() {
        val buryLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.MATCH_PARENT)
        mBuryLayout = LinearLayout(context)
        mBuryLayout.orientation = HORIZONTAL
        mBuryLayout.gravity = Gravity.CENTER_VERTICAL
        mBuryLayout.setPadding(mBuryLeftPadding, 0, mBuryRightPadding, 0)

        mBuryView = AppCompatImageView(context)
        mBuryView.setImageResource(R.drawable.toolbar_down_sel)
        val buryViewLayoutParams = ViewGroup.MarginLayoutParams(mBuryWidth, mBuryHeight)
        mBuryLayout.addView(mBuryView, buryViewLayoutParams)

        addView(mBuryLayout, buryLayoutParams)
        requestLayout()
    }

    /**
     * 点击点赞，做动画
     */
    fun onDiggClick(isDigg: Boolean) {
        mDiggTv.isSelected = isDigg
        mDiggView.cancelAnimation()
        mDiggView.progress = 0f
        if (isDigg) {
            mDiggView.playAnimation()
        }
    }

    /**
     * 点击反对，做动画
     */
    fun onBuryClick(isBury: Boolean) {
        setBuryStatus(isBury)
    }

    /**
     * 设置点赞状态，不做动画
     */
    fun setDiggStatus(isDigg: Boolean) {
        mDiggTv.isSelected = isDigg
        mDiggView.cancelAnimation()
        mDiggView.progress = if (isDigg) 1f else 0f
    }

    /**
     * 设置反对状态，不做动画
     */
    fun setBuryStatus(isBury: Boolean) {
        mBuryView.isSelected = isBury
    }

    fun setDiggClickListener(listener: View.OnClickListener?) {
        mDiggLayout.setOnClickListener(listener)
    }

    fun setBuryClickListener(listener: View.OnClickListener?) {
        mBuryLayout.setOnClickListener(listener)
    }

    fun setText(text: CharSequence?) {
        mDiggTv.text = text
    }

    fun setDiggCount(diggCount: Int) {
        ViewUtils.setTextWithDINStyle(mDiggTv, ViewUtils.getNewDiggText(diggCount), mDigitalTextSize, mWordTextSize)
    }

    fun setText(@StringRes resId: Int) {
        mDiggTv.setText(resId)
    }
}