package com.ss.android.article.wenda.applog

import com.bytedance.article.common.extensions.ifNotEmpty
import com.bytedance.common.utility.StringUtils
import com.ss.android.common.lib.AppLogNewUtils
import org.json.JSONException
import org.json.JSONObject

/**
 * 用于收敛赞同和反对埋点：rt_like和rt_bury
 *
 * @author zhanghao
 * @date 2018/5/16
 * @email zhanghao@bytedance.com
 */
object DiggBuryEventHelper {
    //events
    private const val RT_LIKE                   = "rt_like"
    private const val RT_LIKE_CANCEL            = "rt_like_cancel"
    private const val RT_BURY                   = "rt_bury"
    private const val RT_BURY_CANCEL            = "rt_bury_cancel"

    //params
    private const val PARAM_GROUP_ID            = "group_id"
    private const val PARAM_POSITION            = "position"
    private const val PARAM_SOURCE              = "source"
    private const val PARAM_IS_RANKING          = "is_ranking"
    private const val PARAM_CATEGORY_NAME       = "category_name"
    private const val PARAM_QUESTION_ID         = "question_id"
    private const val PARAM_ANSWER_ID           = "answer_id"
    private const val PARAM_PROFILE_ID          = "profile_id"

    @JvmStatic
    fun onDiggEvent(event: DiggEvent, isDigg: Boolean) {
        val params = getGdExtJb(event.gdExtJson)
        try {
            event.groupId.ifNotEmpty {
                params.put(PARAM_GROUP_ID, it)
            }
            event.position.ifNotEmpty {
                params.put(PARAM_POSITION, it)
            }
            event.source.ifNotEmpty {
                params.put(PARAM_SOURCE, it)
            }
            event.isRanking.ifNotEmpty {
                params.put(PARAM_IS_RANKING, it)
            }
            event.categoryName.ifNotEmpty {
                params.put(PARAM_CATEGORY_NAME, it)
            }
            event.questionId.ifNotEmpty {
                params.put(PARAM_QUESTION_ID, it)
            }
            event.answerId.ifNotEmpty {
                params.put(PARAM_ANSWER_ID, it)
            }
            event.profileId.ifNotEmpty {
                params.put(PARAM_PROFILE_ID, it)
            }
        } catch (e: JSONException) {
        }
        AppLogNewUtils.onEventV3(if (isDigg) RT_LIKE else RT_LIKE_CANCEL, params)
    }

    @JvmStatic
    fun onBuryEvent(event: BuryEvent, isBury: Boolean) {
        val params = getGdExtJb(event.gdExtJson)
        try {
            event.groupId.ifNotEmpty {
                params.put(PARAM_GROUP_ID, it)
            }
            event.position.ifNotEmpty {
                params.put(PARAM_POSITION, it)
            }
            event.source.ifNotEmpty {
                params.put(PARAM_SOURCE, it)
            }
            event.isRanking.ifNotEmpty {
                params.put(PARAM_IS_RANKING, it)
            }
            event.categoryName.ifNotEmpty {
                params.put(PARAM_CATEGORY_NAME, it)
            }
            event.questionId.ifNotEmpty {
                params.put(PARAM_QUESTION_ID, it)
            }
            event.answerId.ifNotEmpty {
                params.put(PARAM_ANSWER_ID, it)
            }
            event.profileId.ifNotEmpty {
                params.put(PARAM_PROFILE_ID, it)
            }
        } catch (e: JSONException) {
        }
        AppLogNewUtils.onEventV3(if (isBury) RT_BURY else RT_BURY_CANCEL, params)
    }

    @JvmStatic
    private fun getGdExtJb(gdExtJson: String?): JSONObject {
        if (StringUtils.isEmpty(gdExtJson)) {
            return JSONObject()
        }

        return try {
            JSONObject(gdExtJson)
        } catch (e: JSONException) {
            JSONObject()
        }
    }

    class DiggEvent {
        var gdExtJson: String? = null
        var groupId: String? = null
        var position: String? = null
        var source: String? = null
        var isRanking: String? = null
        var categoryName: String? = null
        var questionId: String? = null
        var answerId: String? = null
        var profileId: String? = null
    }

    class BuryEvent {
        var gdExtJson: String? = null
        var groupId: String? = null
        var position: String? = null
        var source: String? = null
        var isRanking: String? = null
        var categoryName: String? = null
        var questionId: String? = null
        var answerId: String? = null
        var profileId: String? = null
    }
}