package com.ss.android.article.wenda.digg

import android.content.Context
import android.support.annotation.StringRes
import android.support.v7.widget.AppCompatImageView
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.airbnb.lottie.LottieAnimationView
import com.bytedance.article.common.utils.ViewUtils
import com.bytedance.common.utility.UIUtils
import com.ss.android.article.base.R
import com.ss.android.article.wenda.utils.DINFontUtils

/**
 * 赞同/反对Layout
 * 业务相关性大，暂时不提供高度定制化参数
 * 目前适用：关注频道提问、回答cell，列表页cell，个人主页动态tab提问、回答cell
 *
 * @author zhanghao
 * @date 2018/5/11
 * @email zhanghao@bytedance.com
 */
class DiggBuryLayout : LinearLayout {

    private lateinit var mDiggLayout: LinearLayout
    private lateinit var mDiggView: LottieAnimationView
    private lateinit var mDiggTextTv: TextView
    private lateinit var mDiggCountTv: TextView
    private lateinit var mBuryView: ImageView

    private var mDrawableWidth = UIUtils.dip2Px(context, 28f).toInt()
    private var mDrawableHeight = UIUtils.dip2Px(context, 28f).toInt()
    private var mLayoutHeight = UIUtils.dip2Px(context, 32f).toInt()
    private var mDrawablePadding = UIUtils.dip2Px(context, 7f).toInt()
    private var mDiggLeftPadding = UIUtils.dip2Px(context, 2f).toInt()
    private var mDiggRightPadding = UIUtils.dip2Px(context, 10f).toInt()
    private var mWordTextSize = UIUtils.sp2px(context, 13f).toInt()
    private var mDigitalTextSize = UIUtils.sp2px(context, 14f).toInt()

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)

    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context, attrs, defStyleAttr)
    }

    private fun init(context: Context, attrs: AttributeSet?, defStyleAttr: Int) {
        //允许动画超出边界
        clipToPadding = false
        clipChildren = false
        orientation = HORIZONTAL
        gravity = Gravity.CENTER_VERTICAL

        initDiggLayout(context)
        initBuryLayout()
    }

    private fun initDiggLayout(context: Context) {
        val diggLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, mLayoutHeight)
        mDiggLayout = LinearLayout(context)
        mDiggLayout.orientation = HORIZONTAL
        mDiggLayout.gravity = Gravity.CENTER_VERTICAL
        mDiggLayout.setBackgroundResource(R.drawable.digg_bury_bg_sel)
        mDiggLayout.setPadding(mDiggLeftPadding, 0, mDiggRightPadding, 0)

        mDiggView = LottieAnimationView(context)
        mDiggView.setAnimation("un_dynamic.json", LottieAnimationView.CacheStrategy.Weak)
        val diggViewLayoutParams = ViewGroup.MarginLayoutParams(mDrawableWidth, mDrawableHeight)
        mDiggLayout.addView(mDiggView, diggViewLayoutParams)

        mDiggTextTv = TextView(context)
        mDiggTextTv.setText(R.string.new_digg)
        mDiggTextTv.setTextColor(context.resources.getColorStateList(R.color.c8_c3_selector))
        mDiggTextTv.setTextSize(TypedValue.COMPLEX_UNIT_PX, mWordTextSize.toFloat())
        mDiggTextTv.paint.isFakeBoldText = true
        val diggTextTvLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.WRAP_CONTENT)
        mDiggLayout.addView(mDiggTextTv, diggTextTvLayoutParams)
        (mDiggTextTv.layoutParams as ViewGroup.MarginLayoutParams).leftMargin = UIUtils.dip2Px(context, -1f).toInt()

        mDiggCountTv = TextView(context)
        mDiggCountTv.setTextColor(context.resources.getColorStateList(R.color.c8_c3_selector))
        mDiggCountTv.setTextSize(TypedValue.COMPLEX_UNIT_PX, mDigitalTextSize.toFloat())
        mDiggCountTv.typeface = DINFontUtils.getTypeface(context)
        val diggCountTvLayoutParams = ViewGroup.MarginLayoutParams(ViewGroup.MarginLayoutParams.WRAP_CONTENT, ViewGroup.MarginLayoutParams.WRAP_CONTENT)
        mDiggLayout.addView(mDiggCountTv, diggCountTvLayoutParams)

        addView(mDiggLayout, diggLayoutParams)
        requestLayout()
    }

    private fun initBuryLayout() {
        mBuryView = AppCompatImageView(context)
        mBuryView.setBackgroundResource(R.drawable.digg_bury_bg_sel)
        mBuryView.setImageResource(R.drawable.down_dynamic_sel)
        val buryViewLayoutParams = ViewGroup.MarginLayoutParams(mLayoutHeight, mLayoutHeight)
        addView(mBuryView, buryViewLayoutParams)
        (mBuryView.layoutParams as ViewGroup.MarginLayoutParams).leftMargin = UIUtils.dip2Px(context, 6f).toInt()
        mBuryView.setPadding(mDrawablePadding, mDrawablePadding, mDrawablePadding, mDrawablePadding)
        requestLayout()
    }

    /**
     * 点击点赞，做动画
     */
    fun onDiggClick(isDigg: Boolean) {
        setDiggSelected(isDigg)
        mDiggView.cancelAnimation()
        mDiggView.progress = 0f
        if (isDigg) {
            mDiggView.playAnimation()
        }
    }

    /**
     * 点击反对，做动画
     */
    fun onBuryClick(isBury: Boolean) {
        setBuryStatus(isBury)
    }

    /**
     * 设置点赞状态，不做动画
     */
    fun setDiggStatus(isDigg: Boolean) {
        setDiggSelected(isDigg)
        mDiggView.cancelAnimation()
        mDiggView.progress = if (isDigg) 1f else 0f
    }

    /**
     * 设置反对状态，不做动画
     */
    fun setBuryStatus(isBury: Boolean) {
        mBuryView.isSelected = isBury
    }

    fun setDiggClickListener(listener: View.OnClickListener?) {
        mDiggLayout.setOnClickListener(listener)
    }

    fun setBuryClickListener(listener: View.OnClickListener?) {
        mBuryView.setOnClickListener(listener)
    }

    fun setDiggCount(diggCount: Int) {
        ViewUtils.setTextWithDINStyle(mDiggCountTv, ViewUtils.getDisplayCount(diggCount), mDigitalTextSize, mWordTextSize)
    }

    fun setText(text: CharSequence?) {
        mDiggCountTv.text = text
    }

    fun setText(@StringRes resId: Int) {
        mDiggCountTv.setText(resId)
    }

    fun setDiggCountVisibility(visible: Boolean) {
        mDiggCountTv.visibility = if (visible) {
            VISIBLE
        } else {
            GONE
        }
    }

    private fun setDiggSelected(isDigg: Boolean) {
        mDiggLayout.isSelected = isDigg
        mDiggTextTv.isSelected = isDigg
        mDiggCountTv.isSelected = isDigg
    }
}