package com.ss.android.article.wenda.digg

import android.content.Context
import android.content.res.ColorStateList
import android.graphics.Typeface
import com.bytedance.common.utility.UIUtils
import com.ss.android.article.base.R
import com.ss.android.article.wenda.utils.DINFontUtils

/**
 * @author zhanghao
 * @date 2018/5/10
 * @email zhanghao@bytedance.com
 */
class NewDiggConfig(context: Context) {

    internal var diggTextColor: ColorStateList? = context.resources.getColorStateList(R.color.c8_c3_selector)
    internal var drawablePadding = 0
    internal var drawableWidth = 0
    internal var drawableHeight = 0
    internal var showDiggText = true

    /** Digital：数字状态， Word：文字状态，这两种状态可能有不同的字号，字体等 **/
    internal var digitalTextSize = UIUtils.sp2px(context, 14f).toInt()
    internal var wordTextSize = UIUtils.sp2px(context, 13f).toInt()

    companion object {
        fun configFeed(context: Context) = NewDiggConfig(context)
                .withTextColor(context.resources.getColorStateList(R.color.c8_c3_selector))
                .withDrawablePadding(UIUtils.dip2Px(context, -3f).toInt())
                .withDrawableWH(UIUtils.dip2Px(context, 32f).toInt())
                .withDigitalTextSize(UIUtils.sp2px(context, 14f).toInt())
                .withWordTextSize(UIUtils.sp2px(context, 13f).toInt())

        fun configDetail(context: Context) = NewDiggConfig(context)
                .withDrawableWH(UIUtils.dip2Px(context, 42f).toInt())
                .withShowDiggText(false)
    }

    fun withTextColor(color: ColorStateList): NewDiggConfig {
        diggTextColor = color
        return this
    }

    fun withDrawablePadding(padding: Int): NewDiggConfig {
        drawablePadding = padding
        return this
    }

    fun withDrawableWH(size: Int): NewDiggConfig {
        drawableWidth = size
        drawableHeight = size
        return this
    }

    fun withDrawableWidth(width: Int): NewDiggConfig {
        drawableWidth = width
        return this
    }

    fun withDrawableHeight(height: Int): NewDiggConfig {
        drawableHeight = height
        return this
    }

    fun withTextSize(textSize: Int): NewDiggConfig {
        digitalTextSize = textSize
        wordTextSize = textSize
        return this
    }

    fun withDigitalTextSize(textSize: Int): NewDiggConfig {
        digitalTextSize = textSize
        return this
    }

    fun withWordTextSize(textSize: Int): NewDiggConfig {
        wordTextSize = textSize
        return this
    }

    fun withShowDiggText(showText: Boolean): NewDiggConfig {
        showDiggText = showText
        return this
    }
}