package com.ss.android.article.wenda.recommend

import android.support.v4.content.ContextCompat
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.bytedance.article.common.extensions.ifNotEmpty
import com.bytedance.article.common.extensions.isNotEmpty
import com.bytedance.article.common.impression.ImpressionGroup
import com.bytedance.article.common.impression.ImpressionManager
import com.bytedance.article.common.impression.ImpressionView
import com.bytedance.article.common.model.authentication.UserAuthInfoHelper
import com.bytedance.article.common.utils.ViewUtils
import com.bytedance.common.utility.UIUtils
import com.ss.android.account.app.social.SocialUserConstants
import com.ss.android.account.app.social.SpipeUserMgr
import com.ss.android.account.utils.DebouncingOnClickListener
import com.ss.android.article.base.feature.app.schema.AdsAppActivity
import com.ss.android.article.wenda.utils.AppLogEventConstants
import com.ss.android.article.wenda.utils.WendaUtils
import com.ss.android.common.lib.AppLogNewUtils
import com.ss.android.image.AsyncImageView
import com.ss.android.newmedia.app.AdsAppBaseActivity
import com.ss.android.newmedia.util.AppUtil
import com.ss.android.wenda.api.entity.answerlist.RecommendFollowData
import com.ss.android.wenda.api.entity.common.User
import com.ss.android.article.base.R
import org.json.JSONException
import org.json.JSONObject

/**
 * 推人卡片Adapter
 *
 * @author zhanghao
 * @date 2018/4/16
 * @email zhanghao@bytedance.com
 */
class RecommendFollowAdapter(recommendFollowData: RecommendFollowData) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    companion object {
        const val TYPE_CELL = 0
        const val TYPE_FOOTER = 1

        const val STYLE_DEFAULT = 0
        const val STYLE_DYNAMIC = 1
        const val STYLE_FEED = 2
    }

    private val mData = mutableListOf<User>()
    private val mAddFriendSchema: String?
    private var mImpressionManager: ImpressionManager<*>? = null
    private var mImpressionGroup: ImpressionGroup? = null
    private var mLayoutStyle = STYLE_DEFAULT

    private var mUserCardFollowListener: ((isFollow: Boolean, userId: String?, position: Int) -> Unit)? = null
    private var mFooterClickListener: (() -> Unit)? = null

    init {
        setData(recommendFollowData.reco_follow_users)
        mAddFriendSchema = recommendFollowData.add_friend_schema
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val view: View
        return when (viewType) {
            TYPE_FOOTER -> {
                view = if (mLayoutStyle == STYLE_FEED) {
                    LayoutInflater.from(parent.context).inflate(R.layout.feed_recommend_footer_item, parent, false)
                } else {
                    LayoutInflater.from(parent.context).inflate(R.layout.recommend_footer_item, parent, false)
                }
                RecommendFooterViewHolder(view)
            }
            else -> {
                view = if (mLayoutStyle == STYLE_FEED) {
                    LayoutInflater.from(parent.context).inflate(R.layout.feed_recommend_card_item, parent, false)
                } else {
                    LayoutInflater.from(parent.context).inflate(R.layout.recommend_card_item, parent, false)
                }
                RecommendCardViewHolder(view)
            }
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is RecommendCardViewHolder) {
            val user = mData[position]
            mImpressionManager?.apply {
                mImpressionGroup?.let {
                    bindImpression(it, user, holder.itemView as ImpressionView)
                }
            }
            holder.bindItem(position, user, mLayoutStyle, mUserCardFollowListener)
        } else if (holder is RecommendFooterViewHolder) {
            holder.bindItem(mAddFriendSchema, mFooterClickListener)
        }
    }

    override fun getItemCount() = mData.size + 1

    override fun getItemViewType(position: Int) = if (position == mData.size) TYPE_FOOTER else TYPE_CELL

    /**
     * 填充数据
     */
    fun setData(data: MutableList<User>) {
        mData.clear()
        mData.addAll(data)
        notifyDataSetChanged()
    }

    /**
     * 获取数据
     */
    fun getData(): MutableList<User> {
        val copyData = mutableListOf<User>()
        copyData.addAll(mData)
        return copyData
    }

    /**
     * 添加数据
     */
    fun addData(data: MutableList<User>) {
        mData.addAll(data)
        notifyDataSetChanged()
    }

    /**
     * 添加单条数据
     */
    fun addCell(user: User) {
        addCell(0, user)
    }

    /**
     * 插入单条数据
     */
    fun addCell(index: Int, user: User) {
        mData.add(index, user)
        notifyDataSetChanged()
    }

    /**
     * 删除单条数据
     */
    fun removeCell(user: User) {
        mData.remove(user)
        notifyDataSetChanged()
    }

    fun setUserCardFollowListener(listener: (isFollow: Boolean, userId: String?, position: Int) -> Unit) {
        mUserCardFollowListener = listener
    }

    fun setFooterClickListener(listener: () -> Unit) {
        mFooterClickListener = listener
    }

    fun setImpressionManager(impressionManager: ImpressionManager<*>) {
        mImpressionManager = impressionManager
    }

    fun setImpressionGroup(impressionGroup: ImpressionGroup) {
        mImpressionGroup = impressionGroup
    }

    fun setLayoutStyle(layoutStyle: Int) {
        mLayoutStyle = layoutStyle
    }

    class RecommendCardViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        private val mAvatarView: AsyncImageView = itemView.findViewById(R.id.user_avatar)
        private val mUserView: TextView = itemView.findViewById(R.id.user_name)
        private val mVerifiedView: AsyncImageView = itemView.findViewById(R.id.verified_view)
        private val mUserInfoView: TextView = itemView.findViewById(R.id.user_info)
        private val mFollowBtn: TextView = itemView.findViewById(R.id.follow_btn)
        private val mFollowView: View = itemView.findViewById(R.id.follow_view)

        fun bindItem(position: Int, user: User, layoutStyle: Int, userCardFollowListener: ((isFollow: Boolean, userId: String?, position: Int) -> Unit)?) {
            if (!TextUtils.isEmpty(user.avatar_url)) {
                mAvatarView.setUrl(user.avatar_url)
            }
            mUserView.text = user.uname
            if (user.is_verify == 1) {
                mVerifiedView.visibility = View.VISIBLE
                UserAuthInfoHelper.bindVerifiedImage(mVerifiedView, user.user_auth_info)
            } else {
                mVerifiedView.visibility = View.GONE
            }
            if (user.recommend_reason.isNotEmpty()) {
                mUserInfoView.text = user.recommend_reason
            } else {
                mUserInfoView.text = user.user_intro
            }
            if (layoutStyle == STYLE_FEED) {
                if (user.is_following > 0) {
                    mFollowBtn.setText(R.string.label_entry_followed)
                    mFollowBtn.setTextColor(ContextCompat.getColor(mFollowBtn.context, R.color.c3))
                    mFollowBtn.setCompoundDrawablesWithIntrinsicBounds(0, 0, 0, 0)
                    mFollowBtn.compoundDrawablePadding = 0
                    mFollowView.setBackgroundResource(R.drawable.normal_disabled_button_bg)
                } else {
                    mFollowBtn.setText(R.string.label_entry_follow)
                    mFollowBtn.setTextColor(ContextCompat.getColor(mFollowBtn.context, R.color.c6))
                    mFollowBtn.setCompoundDrawablesWithIntrinsicBounds(R.drawable.followicon_white, 0, 0, 0)
                    mFollowBtn.compoundDrawablePadding = UIUtils.dip2Px(mFollowBtn.context, 4f).toInt()
                    mFollowView.setBackgroundResource(R.drawable.normal_gradient_button_bg)
                }
            } else {
                if (user.is_following > 0) {
                    mFollowBtn.setText(R.string.label_entry_followed)
                    mFollowBtn.setTextColor(ContextCompat.getColor(mFollowBtn.context, R.color.c3))
                    mFollowBtn.setCompoundDrawablesWithIntrinsicBounds(0, 0, 0, 0)
                    mFollowBtn.compoundDrawablePadding = 0
                    mFollowView.setBackgroundResource(R.drawable.msg_follow_btn_bg)
                } else {
                    mFollowBtn.setText(R.string.label_entry_follow)
                    mFollowBtn.setTextColor(ContextCompat.getColor(mFollowBtn.context, R.color.c8))
                    mFollowBtn.setCompoundDrawablesWithIntrinsicBounds(R.drawable.followicon_all, 0, 0, 0)
                    mFollowBtn.compoundDrawablePadding = UIUtils.dip2Px(mFollowBtn.context, 4f).toInt()
                    mFollowView.setBackgroundResource(R.drawable.button_2)
                }
            }
            mFollowView.setOnClickListener(object : DebouncingOnClickListener() {
                override fun doClick(v: View) {
                    // follow the user
                    val isFollow = user.is_following <= 0
                    SpipeUserMgr.getInstance(v.context).followUser(WendaUtils.convertUser(user, SocialUserConstants.ANSWER_LIST_RECOMMEND_CARD),
                            isFollow, null)

                    // add app log
                    logFollowAction(isFollow, user)
                    userCardFollowListener?.invoke(isFollow, user.user_id, position)
                }
            })
            itemView.setOnClickListener(object : DebouncingOnClickListener() {
                override fun doClick(v: View) {
                    if (!TextUtils.isEmpty(user.user_schema)) {
                        val openUrl = AdsAppBaseActivity.tryConvertScheme(user.user_schema)
                        AppUtil.startAdsAppActivity(ViewUtils.getActivity(v), openUrl)
                    }
                }
            })
        }

        private fun logFollowAction(isFollow: Boolean, user: User?) {
            if (user == null) {
                return
            }
            val params = JSONObject()
            try {
                params.put("followed_id", user.user_id)
                params.put("is_verify", user.is_verify)
                params.put("card_position", "feed")
            } catch (e: JSONException) {
            }

            AppLogNewUtils.onEventV3(if (isFollow)
                AppLogEventConstants.EVENT_FOLLOW_CARD_CLICK
            else
                AppLogEventConstants.EVENT_FOLLOW_CARD_CLICK_CANCEL, params)
        }
    }

    class RecommendFooterViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        fun bindItem(addFriendSchema: String?, footerClickListener: (() -> Unit)?) {
            itemView.setOnClickListener(object : DebouncingOnClickListener() {
                override fun doClick(v: View) {
                    addFriendSchema.ifNotEmpty {
                        footerClickListener?.invoke()
                        AdsAppActivity.startAdsAppActivity(v.context, addFriendSchema, null)
                    }
                }
            })
        }
    }
}