package com.ss.android.article.wenda.recommend

import android.content.Context
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.SimpleItemAnimator
import android.text.TextUtils
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import com.bytedance.article.common.extensions.ifEmpty
import com.bytedance.article.common.extensions.ifNotEmpty
import com.bytedance.article.common.impression.ImpressionGroup
import com.bytedance.article.common.impression.ImpressionLinearLayout
import com.bytedance.article.common.impression.ImpressionManager
import com.bytedance.article.common.utils.ViewUtils
import com.bytedance.common.utility.UIUtils
import com.ss.android.account.SpipeData
import com.ss.android.account.app.social.SocialUserConstants
import com.ss.android.account.app.social.SpipeUserMgr
import com.ss.android.account.model.BaseUser
import com.ss.android.account.utils.DebouncingOnClickListener
import com.ss.android.article.base.R
import com.ss.android.article.base.feature.app.schema.AdsAppActivity
import com.ss.android.article.wenda.widget.SlideRecyclerView
import com.ss.android.article.wenda.widget.SpacesItemDecoration
import com.ss.android.wenda.api.entity.answerlist.RecommendFollowData

/**
 * 新推人卡片
 *
 * @author zhanghao
 * @date 2018/4/16
 * @email zhanghao@bytedance.com
 */
class RecommendFollowLayout : ImpressionLinearLayout, SpipeUserMgr.ISpipeUserClient {

    private lateinit var mRecommendTriangleLayout: View
    private lateinit var mRecommendTriangleIv: ImageView
    private lateinit var mRecommendFollowContainer: View
    private lateinit var mRecommendFollowTopLayout: View
    private lateinit var mRecommendTitleTv: TextView
    private lateinit var mRecommendMoreTv: TextView
    private lateinit var mRecyclerView: SlideRecyclerView

    private var mAdapter: RecommendFollowAdapter? = null
    private var mFollowData: RecommendFollowData? = null
    private var mImpressionManager: ImpressionManager<*>? = null
    private var mImpressionGroup: ImpressionGroup? = null
    private lateinit var mItemDecoration: RecyclerView.ItemDecoration
    private var mLayoutStyle = RecommendFollowAdapter.STYLE_DEFAULT

    /** Listeners **/
    private var mUserCardFollowListener: ((isFollow: Boolean, userId: String?, position: Int) -> Unit)? = null
    private var mMoreClickListener: (() -> Unit)? = null
    private var mFooterClickListener: (() -> Unit)? = null

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyleAttr: Int) : super(context, attrs, defStyleAttr)

    override fun onFinishInflate() {
        super.onFinishInflate()
        bindViews()
        initViews()
    }

    private fun bindViews() {
        mRecommendTriangleLayout = findViewById(R.id.recommend_triangle_layout)
        mRecommendTriangleIv = findViewById(R.id.recommend_triangle)
        mRecommendFollowContainer = findViewById(R.id.recommend_follow_container)
        mRecommendFollowTopLayout = findViewById(R.id.recommend_follow_top_layout)
        mRecommendTitleTv = findViewById(R.id.recommend_title_tv)
        mRecommendMoreTv = findViewById(R.id.recommend_more_tv)
        mRecyclerView = findViewById(R.id.recommend_recycler_view)
    }

    private fun initViews() {
        visibility = GONE
        mRecyclerView.layoutManager = LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)
        mRecyclerView.setHasFixedSize(true)
        mItemDecoration = SpacesItemDecoration(UIUtils.dip2Px(context, 6.0f).toInt(),
                UIUtils.dip2Px(context, 16.0f).toInt(), SpacesItemDecoration.HORIZONTAL)
        mRecyclerView.addItemDecoration(mItemDecoration)
        if (mRecyclerView.itemAnimator is SimpleItemAnimator) {
            (mRecyclerView.itemAnimator as SimpleItemAnimator).supportsChangeAnimations = false
        }
        ViewUtils.extendTouchArea(mRecommendMoreTv, 6, true)
    }

    fun bind(data: RecommendFollowData) {
        data.reco_follow_users.ifEmpty {
            visibility = GONE
            return
        }
        visibility = VISIBLE
        mFollowData = data
        val title = data.recommend_title
        val addFriendSchema = data.add_friend_schema
        title.ifNotEmpty {
            mRecommendTitleTv.text = title
        }
        mRecommendMoreTv.setOnClickListener(object : DebouncingOnClickListener() {
            override fun doClick(v: View?) {
                addFriendSchema.ifNotEmpty {
                    mMoreClickListener?.invoke()
                    AdsAppActivity.startAdsAppActivity(context, addFriendSchema, null)
                }
            }
        })
        mAdapter = RecommendFollowAdapter(data)
        mUserCardFollowListener?.let {
            mAdapter?.setUserCardFollowListener(it)
        }
        mFooterClickListener?.let {
            mAdapter?.setFooterClickListener(it)
        }
        mImpressionManager?.let {
            mAdapter?.setImpressionManager(it)
        }
        mImpressionGroup?.let {
            mAdapter?.setImpressionGroup(it)
        }
        mAdapter?.setLayoutStyle(mLayoutStyle)
        mRecyclerView.adapter = mAdapter
        SpipeUserMgr.getInstance(context).addWeakClient(this)
    }

    fun tryBind(data: RecommendFollowData) {
        if (mFollowData == null) {
            bind(data)
        }
    }

    fun releaseResource() {
        mRecyclerView.adapter = null
        mRecyclerView.removeAllViews()
        mFollowData = null
        visibility = GONE
        SpipeUserMgr.getInstance(context).removeWeakClient(this)
    }

    /**
     * 用户卡片关注回调，需要在bind之前设置
     */
    fun setUserCardFollowListener(listener: (isFollow: Boolean, userId: String?, position: Int) -> Unit) {
        mUserCardFollowListener = listener
    }

    /**
     * 查看更多点击回调
     */
    fun setMoreClickListener(listener: () -> Unit) {
        mMoreClickListener = listener
    }

    /**
     * Footer点击回调，需要在bind之前设置
     */
    fun setFooterClickListener(listener: () -> Unit) {
        mFooterClickListener = listener
    }

    /**
     * 设置impression，用于feed，需要在bind之前设置
     */
    fun setImpression(impressionManager: ImpressionManager<*>?, impressionGroup: ImpressionGroup?) {
        mImpressionManager = impressionManager
        mImpressionGroup = impressionGroup
    }

    /**
     * 适应动态卡片(个人主页动态tab)样式，需要在bind之前设置
     */
    fun adaptToDynamicCardStyle() {
        mRecommendTriangleLayout.visibility = GONE
        mRecommendFollowContainer.setBackgroundColor(resources.getColor(R.color.c6))
        mLayoutStyle = RecommendFollowAdapter.STYLE_DYNAMIC
    }

    /**
     * 适应feed卡片样式，需要在bind之前设置
     */
    fun adaptToFeedCardStyle() {
        mRecommendTriangleLayout.visibility = GONE
        mRecommendFollowContainer.setBackgroundColor(resources.getColor(R.color.c6))
        mRecyclerView.removeItemDecoration(mItemDecoration)
        val topLayoutParams = mRecommendFollowTopLayout.layoutParams as MarginLayoutParams
        topLayoutParams.height = UIUtils.dip2Px(context, 36f).toInt()
        mRecommendFollowTopLayout.layoutParams = topLayoutParams
        mRecommendFollowTopLayout.setPadding(0, UIUtils.dip2Px(context, 6f).toInt(), 0, 0)
        val recyclerLayoutParams = mRecyclerView.layoutParams as MarginLayoutParams
        recyclerLayoutParams.bottomMargin = UIUtils.dip2Px(context, 12f).toInt()
        mRecyclerView.layoutParams = recyclerLayoutParams
        mItemDecoration = SpacesItemDecoration(UIUtils.dip2Px(context, 1.0f).toInt(),
                UIUtils.dip2Px(context, 11.0f).toInt(), SpacesItemDecoration.HORIZONTAL)
        mRecyclerView.addItemDecoration(mItemDecoration)
        mLayoutStyle = RecommendFollowAdapter.STYLE_FEED
    }

    /**
     * 调整三角的位置使之与下拉箭头的中心点对齐
     */
    fun adjustTrianglePos(centerPos: Int) {
        val triangleHalfWidth = UIUtils.dip2Px(context, 5.5f).toInt()
        val layoutParams = mRecommendTriangleIv.layoutParams as LinearLayout.LayoutParams
        layoutParams.leftMargin = centerPos - triangleHalfWidth
        mRecommendTriangleIv.layoutParams = layoutParams
    }

    override fun onUserActionDone(error: Int, action: Int, user: BaseUser?) {
        if (user == null || mFollowData == null || error != SpipeData.MSG_POST_OK) {
            return
        }

        val userList = mFollowData?.reco_follow_users
        userList.ifNotEmpty {
            for (followUser in it) {
                if (!TextUtils.equals(followUser.user_id, user.mUserId.toString())) {
                    continue
                }
                followUser.is_following = if (user.isFollowing) 1 else 0
                mAdapter?.notifyItemChanged(it.indexOf(followUser))
                //左滑展示下一个
                if (action == SpipeUserMgr.MSG_ACTION_FOLLOW && TextUtils.equals(user.mNewSource, SocialUserConstants.ANSWER_LIST_RECOMMEND_CARD)) {
                    smoothScrollToShowNextCard(it.indexOf(followUser))
                }
                break
            }
        }
    }

    private fun smoothScrollToShowNextCard(index: Int) {
        if (index < 0 || index >= mAdapter?.itemCount ?: 0 - 1) {
            return
        }
        var firstVisiblePosition = 0
        val layoutManager = mRecyclerView.layoutManager
        if (layoutManager is LinearLayoutManager) {
            firstVisiblePosition = layoutManager.findFirstVisibleItemPosition()
        }
        val currentView = mRecyclerView.getChildAt(index - firstVisiblePosition) ?: return
        val curX = if (currentView.left < 0) 0 else currentView.left
        var rightX = currentView.right + (currentView.layoutParams as RecyclerView.LayoutParams).rightMargin
        val recyclerViewWidth = mRecyclerView.width
        if (rightX > recyclerViewWidth) {
            rightX += rightX - recyclerViewWidth
        }
        mRecyclerView.smoothScrollBy(rightX - curX, 0)
    }

    override fun onUserLoaded(error: Int, user: BaseUser?) {
        //do nothing
    }
}