package com.ss.android.comment

import android.text.SpannableString
import com.bytedance.article.common.ui.richtext.TTRichTextView
import com.bytedance.common.utility.Logger
import com.ss.android.article.base.feature.app.constant.EventConstantV3
import com.ss.android.article.base.utils.link.TouchableSpan
import com.ss.android.common.lib.AppLogNewUtils
import org.json.JSONObject

/**
 * 帖子\评论中富文本的点击事件
 *
 * 帖子富文本包括 @用户名 、 #专题名 、 纯外链三种
 *
 * */
class PostRichTextClickListener(var params: RichTextDataTracker.RichTextTrackParam?) : TTRichTextView.OnDealedSpanListener {

    override fun OnDealSpan(spannableString: SpannableString?) {
        spannableString?.let {
            params?.let { p ->
                val touchableSpans = spannableString.getSpans(0, spannableString.length, TouchableSpan::class.java)
                touchableSpans?.let {
                    for (touchableSpan in it) {
                        if (touchableSpan.shouldSendClickEvents()) {
                            p.type = touchableSpan.linkTypeString
                            // must clone a new instance
                            touchableSpan.addSpanClickListener(TouchableSpanClickListener(p.clone()))
                        }
                    }
                }
            }
        }
    }

    class TouchableSpanClickListener(var params: RichTextDataTracker.RichTextTrackParam)
        : TouchableSpan.ITouchableSpanClick {

        override fun onSpanClick(url: String?) {
            doReport()
        }

        private fun doReport() {
            try {
                val jsonObject = JSONObject()
                jsonObject.putOpt(EventConstantV3.PARAM_CATEGORY_NAME, params.categoryName)
                        .putOpt(EventConstantV3.PARAM_GROUP_ID, params.groupId)
                        .putOpt(EventConstantV3.PARAM_ENTER_FROM, params.enterFrom)
                        .putOpt(EventConstantV3.PARAM_LOG_PB, params.logPb)
                        .putOpt(EventConstantV3.PARAM_GROUP_SOURCE, params.groupSource)
                        .putOpt(EventConstantV3.PARAM_LINK_TYPE, params.type)
                        .putOpt(EventConstantV3.PARAM_SOURCE, params.source)
                var eventName = "richtext_link_click"
                if (RichTextDataTracker.LINK_TYPE_EXTERNAL == params.type) {
                    eventName = "external_link_click"
                }
                AppLogNewUtils.onEventV3(eventName, jsonObject)
            } catch (e: Exception) {
                Logger.d(RichTextDataTracker.TAG, "Json Exception", e)
            }
        }

    }

}