package com.ss.android.comment

import android.support.annotation.StringDef
import com.bytedance.article.common.ui.richtext.TTRichTextView
import org.json.JSONObject


class RichTextDataTracker {
    companion object {
        val TAG: String = RichTextDataTracker::class.java.simpleName

        /**
         * 富文本点击埋点的source属性取值
         * SOURCE_FROM_GROUP 表示是帖子里面的富文本
         * SOURCE_FROM_COMMENT 表示是评论里的富文本
         * SOURCE_FROM_OTHERS 表示其他来源
         * */
        const val SOURCE_FROM_GROUP = "from_group"
        const val SOURCE_FROM_COMMENT = "from_comment"
        const val SOURCE_FROM_OTHERS = "from_others"

        const val LINK_TYPE_UNKNOW = ""
        const val LINK_TYPE_AT = "at_function" //@
        const val LINK_TYPE_TAG = "hashtag" //#
        const val LINK_TYPE_EXTERNAL = "external" //http
        const val LINK_TYPE_IMAGE = "comment_image"  //image

        fun trackRichTextClickIfNeeded(richTextView: TTRichTextView, mLogJsonExtra: JSONObject?, source: String) {
            if (shouldReportRichTextClick(mLogJsonExtra)) {
                richTextView.setDealSpanListener(PostRichTextClickListener(
                        assembleParamsFromParam(mLogJsonExtra, source)))
            }
        }

        /** default value is true
         * */
        fun shouldReportRichTextClick(param: JSONObject?): Boolean {
            return param != null && param.optBoolean(AbsCommentListFragment.COMMENT_REPORT_RICH_TEXT_CLICK, true)
        }

        /** not set type field
         *
         * 最好使用trackRichTextClickIfNeeded， 因为有些情况下是不需要上报的
         * 目前只有帖子（剔除问答，文章，视频，音频，图集）的评论会上报，所以评论都走的 trackRichTextClickIfNeeded
         * */
        fun assembleParamsFromParam(logParam: JSONObject?, source: String): RichTextTrackParam? {
            logParam?.let {
                return RichTextTrackParam().apply {
                    this.categoryName = logParam.optString(AbsCommentListFragment.COMMET_CATEGORY)
                    this.enterFrom = logParam.optString(AbsCommentListFragment.COMMENT_ENTER_FROM)
                    this.groupId = logParam.optLong(AbsCommentListFragment.COMMENT_GROUP_ID)
                    this.groupSource = logParam.optString(AbsCommentListFragment.COMMENT_GROUP_SOURCE)
                    this.logPb = logParam.optString(AbsCommentListFragment.COMMENT_LOGPB)
                    this.source = source
                }
            }
            return null
        }
    }

    @Retention(AnnotationRetention.SOURCE)
    @StringDef(SOURCE_FROM_GROUP, SOURCE_FROM_COMMENT, SOURCE_FROM_OTHERS)
    annotation class SourceFrom

    @Retention(AnnotationRetention.SOURCE)
    @StringDef(LINK_TYPE_UNKNOW, LINK_TYPE_AT, LINK_TYPE_TAG, LINK_TYPE_IMAGE)
    annotation class LinkTypeString

    class RichTextTrackParam {
        var categoryName: String? = null
        @LinkTypeString
        var type: String? = null
        @SourceFrom
        var source: String? = null
        var enterFrom: String? = null
        var groupId: Long? = null
        var groupSource: String? = null
        var logPb: String? = null

        fun clone(): RichTextTrackParam {
            return RichTextTrackParam().apply {
                categoryName = this@RichTextTrackParam.categoryName
                type = this@RichTextTrackParam.type
                source = this@RichTextTrackParam.source
                enterFrom = this@RichTextTrackParam.enterFrom
                groupId = this@RichTextTrackParam.groupId
                groupSource = this@RichTextTrackParam.groupSource
                logPb = this@RichTextTrackParam.logPb
            }
        }
    }
}