package com.ss.android.common.helper

import com.bytedance.article.dex.impl.GsonDependManager
import com.bytedance.common.utility.StringUtils
import com.bytedance.retrofit2.Call
import com.bytedance.retrofit2.Callback
import com.bytedance.retrofit2.SsResponse
import com.bytedance.ttnet.utils.RetrofitUtils
import com.ss.android.account.SpipeData
import com.ss.android.article.base.app.AppData
import com.ss.android.article.base.feature.app.constant.Constants

/**
 * @author: fupeidong at 2018/1/3
 * @email： fupeidong.rd@bytedance.com
 * @function: 用于控制头像单独请求
 */


object UserDecorationManager{

    /**
     * 用于账号切换同步头像挂饰
     */
    fun onAccountRefresh(success: Boolean) {
        if (success && SpipeData.instance().isLogin) {
            if (SpipeData.instance().userId > 0) {
                userDecorationMap.put(SpipeData.instance().userId, SpipeData.instance().userDecoration)
                notifyListeners(SpipeData.instance().userId, SpipeData.instance().userDecoration)
            }
        }
    }

    private val userDecorationMap = LinkedHashMap<Long, String>()
    private val decorationListenerList = HashSet<DecorationListener>()
    private val clearList = ArrayList<Long>()

    private val MAP_SIZE: Int = 1000

    fun fetchUserUrl(userId: Long, url: String?){
        if (userId <= 0) {
            return
        }

        if (isValidUrl(url)) {
            clearDirtyCache()
            userDecorationMap.put(userId, url!!)
        }

        if (isLocalUrlIsValid(userId)) {
            notifyListeners(userId, userDecorationMap[userId]!!)
        } else {
            clearDirtyCache()
            getUserDecorationUrl(userId)
        }

    }

    fun updateLocalUserDecorationUrl(userId: Long, url : String?) {
        if (StringUtils.isEmpty(url) ) {
            userDecorationMap.remove(userId)
        } else {
            userDecorationMap.put(userId, url!!)
        }
    }

    fun getLocalUserDecorationUrl(userId: Long) : String? {
        return userDecorationMap[userId]
    }

    private fun isValidUrl(url : String?) : Boolean{
        return  !StringUtils.isEmpty(url)
    }

    private fun isLocalUrlIsValid(userId : Long) : Boolean {
        return userDecorationMap.containsKey(userId) && !StringUtils.isEmpty(userDecorationMap[userId])
    }

    private fun getUserDecorationUrl(userId: Long) {
        if (!AppData.inst().appSettings.isOrnamentSwitchOpen) {
            return
        }

        val api = RetrofitUtils.createOkService(Constants.API_URL_PREFIX_I, IUserDecorationApi::class.java)
        val callBack = object : Callback<String> {

            override fun onResponse(call: Call<String>?, response: SsResponse<String>?) {
                if (response == null || StringUtils.isEmpty(response.body()) || !response.isSuccessful) {
                    return
                }
                var decorationResponse : UserDecorationEntity? = null
                try {
                    decorationResponse = GsonDependManager.inst().fromJson(response.body(), UserDecorationEntity::class.java)
                } catch (exception: Exception) {
                    return
                }

                if (decorationResponse?.decorationArrayList == null || decorationResponse.decorationArrayList.size == 0) {
                    return
                }
                val decorationList = decorationResponse.decorationArrayList
                decorationList.forEach {
                    if (!StringUtils.isEmpty(it.decoration)) {
                        userDecorationMap.put(userId, it.decoration)
                    } else {
                        userDecorationMap.remove(userId)
                    }
                    notifyListeners(it.userId, it.decoration)
                }
            }

            override fun onFailure(call: Call<String>?, t: Throwable?) {

            }

        }
        api.getDecoration(userId).enqueue(callBack)
    }

    fun registerListener(listener: DecorationListener) {
        decorationListenerList.add(listener)
    }

    fun unregisterListener(listener: DecorationListener) {
        decorationListenerList.remove(listener)
    }

    private fun notifyListeners(userId : Long, ornamentUrl : String) {
        decorationListenerList.forEach {
            it.onDecorationLoaded(userId, ornamentUrl)
        }
    }

    private fun clearDirtyCache() {
        if (userDecorationMap.size < MAP_SIZE) {
            return
        }
        for ((k, _) in userDecorationMap) {
            clearList.add(k)
            if (clearList.size == 100) break
        }
        clearList.forEach { userDecorationMap.remove(it) }
        clearList.clear()
    }

    interface DecorationListener {
        fun onDecorationLoaded(userId : Long, ornamentUrl : String)
    }
}